"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const globals_1 = require("@jest/globals");
const isNestingMock = (value) => Boolean(value) && 'callPath' in value;
const isNestingArgs = (value) => Boolean(value) && Array.isArray(value.args);
const getCalls = (actual, prev = []) => {
    const { calls, results } = actual.mock;
    return calls
        .map((call, index) => ({
        args: call,
        result: results[index],
    }))
        .flatMap(call => {
        const actualArgs = [...prev, call.args];
        if (call.result.type === 'return' && isNestingMock(call.result.value)) {
            const result = getCalls(call.result.value, actualArgs);
            if (result.length)
                return result;
        }
        return [actualArgs];
    });
};
const getMatch = (context, actual, expected) => {
    const sharedLength = Math.min(actual.length, expected.length);
    let index = 0;
    while (index < sharedLength) {
        const isMatch = context.equals(actual[index], expected[index]);
        if (!isMatch)
            break;
        index++;
    }
    return actual.slice(0, index);
};
const getMatches = (context, actual, expected) => actual.map(call => getMatch(context, call, expected)).filter(match => match.length);
const printCall = (args) => `fn(${args.map(call => call.join(', ')).join(')(')})`;
function toHaveBeenNestedCalledWith(actual, args) {
    if (!isNestingMock(actual)) {
        throw new Error('Actual must be a Nesting mock.');
    }
    const expected = isNestingArgs(args) ? args.args : args;
    if (!Array.isArray(expected)) {
        throw new Error('Args must be of type NestingArgs or any[][].');
    }
    const calls = getCalls(actual);
    const matches = getMatches(this, calls, expected);
    if (!matches.length) {
        return {
            message: () => `Expected the nested function to have been called
Expected: ${printCall(expected)}
Actual: Number of calls: 0`,
            pass: false,
        };
    }
    const isMatch = matches.find(m => m.length === expected.length);
    if (isMatch) {
        return {
            message: () => `Expected calls not to match ${printCall(expected)}`,
            pass: true,
        };
    }
    return {
        message: () => `Expected calls to match
Expected: ${printCall(expected)}
Actual:
  ${calls.map(m => printCall(m)).join('\n  ')}`,
        pass: false,
    };
}
globals_1.expect.extend({
    toHaveBeenNestedCalledWith,
    toBeNestedCalledWith: toHaveBeenNestedCalledWith,
});
//# sourceMappingURL=matcher.js.map